@@ -21,9 +21,13 @@
 #include "Filter.h"
 
 // Qt
+#include <QDebug>
 #include <QAction>
 #include <QApplication>
 #include <QtGui/QClipboard>
+#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtCore/QMimeDatabase>
 #include <QtCore/QString>
 #include <QtCore/QTextStream>
 #include <QtCore/QUrl>
@@ -33,6 +37,7 @@
 #include <KRun>
 
 // Konsole
+#include "Session.h"
 #include "TerminalCharacterDecoder.h"
 #include "konsole_wcwidth.h"
 
@@ -295,8 +300,11 @@ RegExpFilter::RegExpFilter()
 {
 }
 
-RegExpFilter::HotSpot::HotSpot(int startLine, int startColumn, int endLine, int endColumn)
+RegExpFilter::HotSpot::HotSpot(int startLine, int startColumn,
+                               int endLine, int endColumn,
+                               const QStringList& capturedTexts)
     : Filter::HotSpot(startLine, startColumn, endLine, endColumn)
+    , _capturedTexts(capturedTexts)
 {
     setType(Marker);
 }
@@ -305,20 +313,17 @@ void RegExpFilter::HotSpot::activate(QObject*)
 {
 }
 
-void RegExpFilter::HotSpot::setCapturedTexts(const QStringList& texts)
-{
-    _capturedTexts = texts;
-}
 QStringList RegExpFilter::HotSpot::capturedTexts() const
 {
     return _capturedTexts;
 }
 
-void RegExpFilter::setRegExp(const QRegExp& regExp)
+void RegExpFilter::setRegExp(const QRegularExpression &regExp)
 {
     _searchText = regExp;
+    _searchText.optimize();
 }
-QRegExp RegExpFilter::regExp() const
+QRegularExpression RegExpFilter::regExp() const
 {
     return _searchText;
 }
@@ -328,56 +333,51 @@ QRegExp RegExpFilter::regExp() const
 }*/
 void RegExpFilter::process()
 {
-    int pos = 0;
     const QString* text = buffer();
 
     Q_ASSERT(text);
 
-    // ignore any regular expressions which match an empty string.
-    // otherwise the while loop below will run indefinitely
-    if (_searchText.isEmpty())
+    if (!_searchText.isValid()) {
         return;
+    }
 
-    while (pos >= 0) {
-        pos = _searchText.indexIn(*text, pos);
+    QRegularExpressionMatchIterator iterator(_searchText.globalMatch(*text));
+    while (iterator.hasNext()) {
+        QRegularExpressionMatch match(iterator.next());
 
-        if (pos >= 0) {
-            int startLine = 0;
-            int endLine = 0;
-            int startColumn = 0;
-            int endColumn = 0;
+        int startLine = 0;
+        int endLine = 0;
+        int startColumn = 0;
+        int endColumn = 0;
 
-            getLineColumn(pos, startLine, startColumn);
-            getLineColumn(pos + _searchText.matchedLength(), endLine, endColumn);
+        getLineColumn(match.capturedStart(), startLine, startColumn);
+        getLineColumn(match.capturedEnd(), endLine, endColumn);
 
-            RegExpFilter::HotSpot* spot = newHotSpot(startLine, startColumn,
-                                          endLine, endColumn);
-            spot->setCapturedTexts(_searchText.capturedTexts());
-
-            addHotSpot(spot);
-            pos += _searchText.matchedLength();
-
-            // if matchedLength == 0, the program will get stuck in an infinite loop
-            if (_searchText.matchedLength() == 0)
-                pos = -1;
+        RegExpFilter::HotSpot* spot = newHotSpot(startLine, startColumn,
+                                                 endLine, endColumn, match.capturedTexts());
+        if (!spot) {
+            continue;
         }
+
+        addHotSpot(spot);
     }
 }
 
 RegExpFilter::HotSpot* RegExpFilter::newHotSpot(int startLine, int startColumn,
-        int endLine, int endColumn)
+        int endLine, int endColumn, const QStringList& capturedTexts)
 {
     return new RegExpFilter::HotSpot(startLine, startColumn,
-                                     endLine, endColumn);
+                                     endLine, endColumn, capturedTexts);
 }
-RegExpFilter::HotSpot* UrlFilter::newHotSpot(int startLine, int startColumn, int endLine,
-        int endColumn)
+RegExpFilter::HotSpot* UrlFilter::newHotSpot(int startLine, int startColumn,
+        int endLine, int endColumn, const QStringList& capturedTexts)
 {
     return new UrlFilter::HotSpot(startLine, startColumn,
-                                  endLine, endColumn);
+                                  endLine, endColumn, capturedTexts);
 }
-UrlFilter::HotSpot::HotSpot(int startLine, int startColumn, int endLine, int endColumn)
-    : RegExpFilter::HotSpot(startLine, startColumn, endLine, endColumn)
+UrlFilter::HotSpot::HotSpot(int startLine, int startColumn, int endLine, int endColumn,
+        const QStringList& capturedTexts)
+    : RegExpFilter::HotSpot(startLine, startColumn, endLine, endColumn, capturedTexts)
     , _urlObject(new FilterObject(this))
 {
     setType(Link);
@@ -387,12 +387,13 @@ UrlFilter::HotSpot::UrlType UrlFilter::HotSpot::urlType() const
 {
     const QString url = capturedTexts().at(0);
 
-    if (FullUrlRegExp.exactMatch(url))
+    if (FullUrlRegExp.match(url).hasMatch()) {
         return StandardUrl;
-    else if (EmailAddressRegExp.exactMatch(url))
+    } else if (EmailAddressRegExp.match(url).hasMatch()) {
         return Email;
-    else
+    } else {
         return Unknown;
+    }
 }
 
 void UrlFilter::HotSpot::activate(QObject* object)
@@ -431,14 +432,17 @@ void UrlFilter::HotSpot::activate(QObject* object)
 //regexp matches:
 // full url:
 // protocolname:// or www. followed by anything other than whitespaces, <, >, ' or ", and ends before whitespaces, <, >, ', ", ], !, ), :, comma and dot
-const QRegExp UrlFilter::FullUrlRegExp("(www\\.(?!\\.)|[a-z][a-z0-9+.-]*://)[^\\s<>'\"]+[^!,\\.\\s<>'\"\\]\\)\\:]");
+const QRegularExpression UrlFilter::FullUrlRegExp("(www\\.(?!\\.)|[a-z][a-z0-9+.-]*://)[^\\s<>'\"]+[^!,\\.\\s<>'\"\\]\\)\\:]",
+                                                  QRegularExpression::OptimizeOnFirstUsageOption);
 // email address:
 // [word chars, dots or dashes]@[word chars, dots or dashes].[word chars]
-const QRegExp UrlFilter::EmailAddressRegExp("\\b(\\w|\\.|-)+@(\\w|\\.|-)+\\.\\w+\\b");
+const QRegularExpression UrlFilter::EmailAddressRegExp("\\b(\\w|\\.|-)+@(\\w|\\.|-)+\\.\\w+\\b",
+                                                       QRegularExpression::OptimizeOnFirstUsageOption);
 
 // matches full url or email address
-const QRegExp UrlFilter::CompleteUrlRegExp('(' + FullUrlRegExp.pattern() + '|' +
-        EmailAddressRegExp.pattern() + ')');
+const QRegularExpression UrlFilter::CompleteUrlRegExp('(' + FullUrlRegExp.pattern() + '|' +
+                                                      EmailAddressRegExp.pattern() + ')',
+                                                      QRegularExpression::OptimizeOnFirstUsageOption);
 
 UrlFilter::UrlFilter()
 {
@@ -484,3 +488,75 @@ QList<QAction*> UrlFilter::HotSpot::actions()
     return actions;
 }
 
+/**
+  * File Filter - Construct a filter that works on local file paths using the
+  * posix portable filename character set combined with KDE's mimetype filename
+  * extension blob patterns.
+  * http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap03.html#tag_03_267
+  */
+
+RegExpFilter::HotSpot* FileFilter::newHotSpot(int startLine, int startColumn, int endLine,
+        int endColumn, const QStringList& capturedTexts)
+{
+    if (!_session) {
+        qWarning() << "Trying to create new hot spot without session!";
+        return nullptr;
+    }
+
+    FileFilter::HotSpot *spot = nullptr;
+    QDir dir(_session->currentWorkingDirectory());
+    QFileInfo file(dir, capturedTexts.first());
+    if (file.exists())
+        spot = new FileFilter::HotSpot(startLine, startColumn, endLine, endColumn, capturedTexts, file);
+    return spot;
+}
+
+FileFilter::HotSpot::HotSpot(int startLine, int startColumn, int endLine, int endColumn, const QStringList& capturedTexts, const QFileInfo& file)
+    : RegExpFilter::HotSpot(startLine, startColumn, endLine, endColumn, capturedTexts)
+    , _fileObject(new FilterObject(this))
+    , _file(file)
+{
+    setType(Link);
+}
+
+void FileFilter::HotSpot::activate(QObject*)
+{
+    QString file = _file.absoluteFilePath();
+    new KRun(QUrl::fromLocalFile(file), QApplication::activeWindow());
+}
+
+FileFilter::FileFilter(Session* session)
+  : _session(session)
+{
+    QStringList patterns;
+    QMimeDatabase mimeDatabase;
+    for (const QMimeType &mimeType : mimeDatabase.allMimeTypes()) {
+        patterns.append(mimeType.globPatterns());
+    }
+
+    patterns.removeDuplicates();
+    patterns.replaceInStrings("*", "");
+    patterns = patterns.filter(QRegularExpression("^[A-Za-z0-9\\._-]+$"));
+    patterns.replaceInStrings(".", "\\.");
+
+    QString regex("(\\b|/)+[A-Za-z0-9\\._\\-/]+(");
+    regex.append(patterns.join("|"));
+    regex.append("){1}\\b");
+    setRegExp(QRegularExpression(regex));
+}
+
+FileFilter::HotSpot::~HotSpot()
+{
+    delete _fileObject;
+}
+
+QList<QAction*> FileFilter::HotSpot::actions()
+{
+    QAction* openAction = new QAction(_fileObject);
+    openAction->setText(i18n("Open File"));
+    QObject::connect(openAction , SIGNAL(triggered()) , _fileObject , SLOT(activated()));
+    QList<QAction*> actions;
+    actions << openAction;
+    return actions;
+}
+
