@@ -24,6 +24,7 @@
 
 // Qt
 #include <QtCore/QTextCodec>
+#include <QtCore/QRegularExpression>
 
 // KDE
 #include <KLocalizedString>
@@ -77,6 +78,7 @@ const Profile::PropertyInfo Profile::DefaultPropertyNames[] = {
     , { ColorScheme , "colors" , 0 , QVariant::String }
     , { AntiAliasFonts, "AntiAliasFonts" , APPEARANCE_GROUP , QVariant::Bool }
     , { BoldIntense, "BoldIntense", APPEARANCE_GROUP, QVariant::Bool }
+    , { UseFontLineCharacters, "UseFontLineChararacters", APPEARANCE_GROUP, QVariant::Bool }
     , { LineSpacing , "LineSpacing" , APPEARANCE_GROUP , QVariant::Int }
 
     // Keyboard
@@ -89,6 +91,7 @@ const Profile::PropertyInfo Profile::DefaultPropertyNames[] = {
     , { ScrollFullPage , "ScrollFullPage" , SCROLLING_GROUP , QVariant::Bool }
 
     // Terminal Features
+    , { EnableUrlHints , "EnableUrlHints" , TERMINAL_GROUP , QVariant::Bool }
     , { BlinkingTextEnabled , "BlinkingTextEnabled" , TERMINAL_GROUP , QVariant::Bool }
     , { FlowControlEnabled , "FlowControlEnabled" , TERMINAL_GROUP , QVariant::Bool }
     , { BidiRenderingEnabled , "BidiRenderingEnabled" , TERMINAL_GROUP , QVariant::Bool }
@@ -149,6 +152,7 @@ FallbackProfile::FallbackProfile()
     // non-directory file name
     setProperty(Path, "FALLBACK/");
     setProperty(Command, qgetenv("SHELL"));
+    // See Pty.cpp on why Arguments is populated
     setProperty(Arguments, QStringList() << qgetenv("SHELL"));
     setProperty(Icon, "utilities-terminal");
     setProperty(Environment, QStringList() << QStringLiteral("TERM=xterm"));
@@ -172,6 +176,7 @@ FallbackProfile::FallbackProfile()
     setProperty(ScrollFullPage, false);
 
     setProperty(FlowControlEnabled, true);
+    setProperty(EnableUrlHints, false);
     setProperty(BlinkingTextEnabled, true);
     setProperty(UnderlineLinksEnabled, true);
     setProperty(OpenLinksByDirectClickEnabled, false);
@@ -195,6 +200,7 @@ FallbackProfile::FallbackProfile()
     setProperty(DefaultEncoding, QString(QTextCodec::codecForLocale()->name()));
     setProperty(AntiAliasFonts, true);
     setProperty(BoldIntense, true);
+    setProperty(UseFontLineCharacters, false);
 
     setProperty(WordCharacters, ":@-./_~?&=%+#");
 
@@ -314,20 +320,14 @@ QHash<Profile::Property, QVariant> ProfileCommandParser::parse(const QString& in
     // where 'property' is a word consisting only of characters from A-Z
     // where 'value' is any sequence of characters other than a semi-colon
     //
-    static QRegExp regExp("([a-zA-Z]+)=([^;]+)");
+    static const QRegularExpression regExp("([a-zA-Z]+)=([^;]+)");
 
-    int offset = 0;
-    while (regExp.indexIn(input, offset) != -1) {
-        if (regExp.capturedTexts().count() == 3) {
-            Profile::Property property = Profile::lookupByName(
-                                             regExp.capturedTexts()[1]);
-            const QString value = regExp.capturedTexts()[2];
-            changes.insert(property, value);
-        }
-
-        offset = input.indexOf(';', offset) + 1;
-        if (offset == 0)
-            break;
+    QRegularExpressionMatchIterator iterator(regExp.globalMatch(input));
+    while (iterator.hasNext()) {
+        QRegularExpressionMatch match(iterator.next());
+        Profile::Property property = Profile::lookupByName(match.captured(1));
+        const QString value = match.captured(2);
+        changes.insert(property, value);
     }
 
     return changes;
