@@ -61,6 +61,7 @@
 using namespace Konsole;
 
 int Session::lastSessionId = 0;
+static bool show_disallow_certain_dbus_methods_message = true;
 
 // HACK This is copied out of QUuid::createUuid with reseeding forced.
 // Required because color schemes repeatedly seed the RNG...
@@ -76,7 +77,7 @@ QUuid createUuid()
         } while ((max = max >> 1));
     }
 
-    qsrand(uint(QDateTime::currentDateTime().toTime_t()));
+    qsrand(uint(QDateTime::currentDateTimeUtc().toTime_t()));
     qrand(); // Skip first
 
     QUuid result;
@@ -186,7 +187,8 @@ void Session::openTeletype(int fd)
     connect(_shellProcess, static_cast<void(Pty::*)(int,QProcess::ExitStatus)>(&Konsole::Pty::finished), this, &Konsole::Session::done);
 
     // emulator size
-    connect(_emulation, &Konsole::Emulation::imageSizeChanged, this, &Konsole::Session::updateWindowSize);
+    // Use a direct connection to ensure that the window size is set before it runs
+    connect(_emulation, &Konsole::Emulation::imageSizeChanged, this, &Konsole::Session::updateWindowSize, Qt::DirectConnection);
     connect(_emulation, &Konsole::Emulation::imageSizeInitialized, this, &Konsole::Session::run);
 }
 
@@ -267,6 +269,10 @@ void Session::setInitialWorkingDirectory(const QString& dir)
 
 QString Session::currentWorkingDirectory()
 {
+    if (_reportedWorkingUrl.isValid() && _reportedWorkingUrl.isLocalFile()) {
+        return _reportedWorkingUrl.path();
+    }
+
     // only returned cached value
     if (_currentWorkingDir.isEmpty())
         updateWorkingDirectory();
@@ -547,6 +553,12 @@ void Session::setUserTitle(int what, const QString& caption)
         }
     }
 
+    if (what == CurrentDirectory) {
+        _reportedWorkingUrl = QUrl::fromUserInput(caption);
+        emit currentDirectoryChanged(currentWorkingDirectory());
+        modified = true;
+    }
+
     if (what == ProfileChange) {
         emit profileChangeCommandReceived(caption);
         return;
@@ -586,14 +598,24 @@ void Session::silenceTimerDone()
     //when any of the views of the session becomes active
 
     //FIXME: Make message text for this notification and the activity notification more descriptive.
-    if (_monitorSilence) {
-        KNotification::event("Silence", i18n("Silence in session '%1'", _nameTitle), QPixmap(),
-                             QApplication::activeWindow(),
-                             KNotification::CloseWhenWidgetActivated);
-        emit stateChanged(NOTIFYSILENCE);
-    } else {
+    if (!_monitorSilence) {
         emit stateChanged(NOTIFYNORMAL);
+        return;
     }
+
+    bool hasFocus = false;
+    for (TerminalDisplay *display : _views) {
+        if (display->hasFocus()) {
+            hasFocus = true;
+            break;
+        }
+    }
+
+    KNotification::event(hasFocus ? "Silence" : "SilenceHidden",
+            i18n("Silence in session '%1'", _nameTitle), QPixmap(),
+            QApplication::activeWindow(),
+            KNotification::CloseWhenWidgetActivated);
+    emit stateChanged(NOTIFYSILENCE);
 }
 
 void Session::activityTimerDone()
@@ -640,8 +662,18 @@ void Session::activityStateSet(int state)
     if (state == NOTIFYBELL) {
         emit bellRequest(i18n("Bell in session '%1'", _nameTitle));
     } else if (state == NOTIFYACTIVITY) {
+        // Don't notify if the terminal is active
+        bool hasFocus = false;
+        for (TerminalDisplay *display : _views) {
+            if (display->hasFocus()) {
+                hasFocus = true;
+                break;
+            }
+        }
+
         if (_monitorActivity  && !_notifiedActivity) {
-            KNotification::event("Activity", i18n("Activity in session '%1'", _nameTitle), QPixmap(),
+            KNotification::event(hasFocus ? "Activity" : "ActivityHidden",
+                                 i18n("Activity in session '%1'", _nameTitle), QPixmap(),
                                  QApplication::activeWindow(),
                                  KNotification::CloseWhenWidgetActivated);
 
@@ -739,9 +771,9 @@ void Session::reportBackgroundColor(const QColor& c)
 {
     #define to65k(a) (QString("%1").arg((int)(a*0xFFFF), 4, 16, QChar('0')))
     QString msg = "\033]11;rgb:"
-                + to65k(c.redF())   + "/"
-                + to65k(c.greenF()) + "/"
-                + to65k(c.blueF())  + "\a";
+                + to65k(c.redF())   + '/'
+                + to65k(c.greenF()) + '/'
+                + to65k(c.blueF())  + '\a';
     _emulation->sendString(msg.toUtf8());
     #undef to65k
 }
@@ -790,6 +822,15 @@ bool Session::closeInNormalWay()
         return true;
     }
 
+    _shellProcess->sendEof();
+
+    if (_shellProcess->waitForFinished(1000)) {
+        return true;
+    }
+
+    qWarning() << "shell did not close, sending SIGHUP";
+
+    // We tried asking nicely, ask a bit less nicely
     if (kill(SIGHUP)) {
         return true;
     } else {
@@ -812,14 +853,31 @@ bool Session::closeInForceWay()
     }
 }
 
+void Session::sendTextToTerminal(const QString& text, const QChar& eol) const
+{
+    _emulation->sendText(text + eol);
+}
+
+// Only D-Bus calls this function (via SendText or runCommand)
 void Session::sendText(const QString& text) const
 {
+#if not defined(REMOVE_SENDTEXT_RUNCOMMAND_DBUS_METHODS)
+    if (show_disallow_certain_dbus_methods_message) {
+
+        KNotification::event(KNotification::Warning, "Konsole D-Bus Warning",
+            i18n("The D-Bus methods sendText/runCommand were just used.  There are security concerns about allowing these methods to be public.  If desired, these methods can be changed to internal use only by re-compiling Konsole. <p>This warning will only show once for this Konsole instance.</p>"));
+
+        show_disallow_certain_dbus_methods_message = false;
+    }
+#endif
+
     _emulation->sendText(text);
 }
 
+// Only D-Bus calls this function
 void Session::runCommand(const QString& command) const
 {
-    _emulation->sendText(command + '\n');
+    sendText(command + '\n');
 }
 
 void Session::sendMouseEvent(int buttons, int column, int line, int eventType)
@@ -949,7 +1007,8 @@ void Session::updateSessionProcessInfo()
     if (!_sessionProcessInfo ||
             (processId() != 0 && processId() != _sessionProcessInfo->pid(&ok))) {
         delete _sessionProcessInfo;
-        _sessionProcessInfo = ProcessInfo::newInstance(processId());
+        _sessionProcessInfo = ProcessInfo::newInstance(processId(),
+                    tabTitleFormat(Session::LocalTabTitle));
         _sessionProcessInfo->setUserHomeDir();
     }
     _sessionProcessInfo->update();
@@ -962,7 +1021,8 @@ bool Session::updateForegroundProcessInfo()
     const int foregroundPid = _shellProcess->foregroundProcessGroup();
     if (foregroundPid != _foregroundPid) {
         delete _foregroundProcessInfo;
-        _foregroundProcessInfo = ProcessInfo::newInstance(foregroundPid);
+        _foregroundProcessInfo = ProcessInfo::newInstance(foregroundPid,
+                    tabTitleFormat(Session::LocalTabTitle));
         _foregroundPid = foregroundPid;
     }
 
@@ -986,6 +1046,7 @@ QString Session::getDynamicTitle()
 {
     // update current directory from process
     updateWorkingDirectory();
+
     ProcessInfo* process = getProcessInfo();
 
     // format tab titles using process info
@@ -1003,6 +1064,10 @@ QString Session::getDynamicTitle()
 
 QUrl Session::getUrl()
 {
+    if (_reportedWorkingUrl.isValid()) {
+        return _reportedWorkingUrl;
+    }
+
     QString path;
 
     updateSessionProcessInfo();
